#!/bin/bash

 ##############################################################################
 # Filename: bash/functions.sh
 #
 # This file is part of the JAS project.
 # Copyright 2003-2015 by Matthijs Schuurman
 #
 # The JAS project is free software under the terms of the
 # GNU Lesser General Public License version 3 or any later version.
 #
 # For information about the license please refer to:
 # LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 ##############################################################################


function ifs()
{
 local NEWIFS=$1

 if [ "$CURIFS" == "" ]; then #IFS (internal field separator) not yet stored
  CURIFS="$IFS"
 fi

 if [ "$CURIFS" == "$IFS" ]; then #switch to new
  IFS="$NEWIFS"
 else #switch back
  IFS="$CURIFS"
 fi
}

function title()
{
 local TITLE=$1
 local VERSION=$2

 echo
 echo -en '\e[1;30m'"$TITLE"'\e[0m'
 if [ "$VERSION" != "" ]; then
  echo -en " v$VERSION"
 fi
 echo
 echo
}

function action()
{
 local ACTION=$1
 local TASK=$2

 echo -en "$ACTION"
 if [ "$TASK" != "" ]; then
  echo -en " ["'\e[1;30m'"$TASK"'\e[0m'"]"
 fi
 echo -en "... "
}

function status()
{
 local ERROR=$1
 local TEXT=$2

 if [[ "$ERROR" == +([0-9]) ]] ; then
 {
  if [ $ERROR -eq 0 ]; then
   local ERROR="done"
  else
   local ERROR="failed"
  fi
 }
 fi

 if [ "$ERROR" == "failed" ]; then
  local COLOR="31"
 elif [ "$ERROR" == "done" ]; then
  local COLOR="32"
 elif [ "$ERROR" == "skipped" ]; then
  local COLOR="34"
 else
  local COLOR="33"
 fi

 echo -e "["'\e[1;'$COLOR'm'"$ERROR"'\e[0m'"]"

 if [ "$ERROR" != "done" -a "$TEXT" ] ; then
  echo -e "Error: "'\e[1;33m'"$TEXT"'\e[0m'
 fi

 if [ "$ERROR" == "failed" ]; then
  return 1
 elif [ "$ERROR" == "done" ]; then
  return 0
 fi

 return 2
}


function getDirectories()
{
 local TYPE=$1
 local SUBDIRECTORIES=$2
 local DIRECTORIES

 ifs #leave output alone

 if [ "$TYPE" == "" ]; then #JAS and plugin directories
 {
  getDirectories "jas"
  getDirectories "plugins"

  ifs #restore IFS
  return 0
 }
 elif [ "$TYPE" == "hg" ]; then #.hg directories
  DIRECTORIES="`find . -type d -name \".hg\" 2>/dev/null | sed -e "s/\.hg$//" | sort`"
 elif [ "$TYPE" == "jas" ]; then #JAS directories
  DIRECTORIES="./
./bash/
./files/
./images/
./include/
./jas/
./script/
./api/
./project/jas/"
 elif [ "$TYPE" == "projects" ]; then #project directories
  DIRECTORIES="`find ./project/ -type d -maxdepth 1 2>/dev/null | grep -v "project/$" | sort`"
 elif [ "`echo $TYPE | grep \"/\"`" != "" ]; then #slashes: path
  DIRECTORIES="$TYPE" #set directory
 elif [ "$TYPE" == "license" ]; then #directories with license
  DIRECTORIES="`find ./ -type f -name LICENSE.txt 2>/dev/null | sed -e "s/LICENSE.txt//" | sort`"
 elif [ "$TYPE" == "plugins" ]; then #All plugin directories
  DIRECTORIES="`find . -type d -path "*/plugins/*" -a -not -path \"*/.hg/*\" 2>/dev/null | grep "plugins/[^/]\+$" | sort`"
 else #specific plugin directory
  DIRECTORIES="`find . -type d -path \"*plugins/$TYPE\" -a -not -path \"*/.hg/*\" 2>/dev/null`"
 fi

 #get all subdirectories
 if [ "$SUBDIRECTORIES" != "" ]; then
 {
  ifs #restore IFS
  local TMP=($DIRECTORIES)
  ifs #leave output alone

  DIRECTORIES=""
  for DIR in ${TMP[@]}; do
  {
   if [ "$DIR" == "./" ]; then #don't dive into ./
    DIRECTORIES="$DIRECTORIES$DIR
"
   else
   {
    local TMP="`find \"$DIR\" -type d -not -path \"*/.hg*\" 2>/dev/null | sort`"

    if [ "$TMP" != "" ]; then
     DIRECTORIES="$DIRECTORIES$TMP
"
    fi
   }
   fi
  }
  done

  #remove empty line at end
  if [ "${DIRECTORIES: -1}" == "
" ]; then
   DIRECTORIES=${DIRECTORIES:0: -1}
  fi
 }
 fi


 echo $DIRECTORIES

 ifs #restore IFS
 return 0
}

function getFiles()
{
 local TYPE=$1
 local REGEX=$2
 local FILES

 DIRECTORIES=(`getDirectories $TYPE true`)

 ifs #leave output alone

 #get all subdirectories
 for DIR in ${DIRECTORIES[@]}; do
 {
  if [ "$REGEX" != "" ]; then
   local TMP="`find \"$DIR\" -maxdepth 1 -type f -regex $REGEX -not -path \"*.hg?*\" -not -path \"*.sw?\" -not -path \"*JAS-*.zip\" -not -path \"*project/*/cache/*/*\" -not -path \"*project/*/include/plugins/status.txt 2>/dev/null | sort`"
  else
   local TMP="`find \"$DIR\" -maxdepth 1 -type f -not -path \"*.hg?*\" -not -path \"*.sw?\" -not -path \"*JAS-*.zip\" -not -path \"*project/*/cache/*/*\" -not -path \"*project/*/include/plugins/status.txt\" 2>/dev/null | sort`"
  fi

  if [ "$TMP" != "" ]; then
   FILES="$FILES$TMP
"
  fi
 }
 done

 if [ "$TYPE" == "jas" ]; then
  FILES="${FILES}./project/index.php
"
 fi

 #remove empty line at end
 if [ "${FILES: -1}" == "
" ]; then
  FILES=${FILES:0: -1}
 fi


 echo $FILES

 ifs #restore IFS
 return 0
}


function generateChangelog()
{
 local DIRECTORY=$1
 local CURDIR=`pwd`
 local OUTPUT

 #check if .hg directory exists
 if [ ! -d "$DIRECTORY/.hg/" ]; then
  echo "No .hg directory found in: \"$DIRECTORY\""
  return 1
 fi

 ifs #leave output alone
 cd "$DIRECTORY" #go to hg directory

 #get changelog
 OUTPUT=`hg log --style "$CURDIR/bash/changelog.hg" 2>&1`

 if [ $? != 0 ]; then #failed
  echo $OUTPUT
  cd "$CURDIR" #go back to current directory
  ifs #restore IFS
  return 1
 fi

 cd "$CURDIR" #go back to current directory

 #manipulate changelog and output
 echo $OUTPUT \
  | perl -pane "BEGIN{undef $/} s/(\n+\t[0-9\-]+\n)?\t\* Added tag.*//g" \
  | perl -pane "BEGIN{undef $/} s/Version tip\n+Version/Version/" \
  | perl -pane "s/Version tip/Development version/" \

 ifs #restore IFS
 return 0
}


function getLicense()
{
 local FILE=$1

 if [ ! -f "$FILE" ]; then
  echo "License file: $FILE not found"
  return 1
 fi

 EXT=`echo "$FILE" | perl -pane "s/.*\.([a-z]+)/\1/"` #get file extension

 ifs #leave output alone

 if [ "$EXT" == "php" ]; then #PHP file
  LICENSE="`perl -0777 -pane \"s!.*\<\?\n*(\s*/\*\*\*.+?\*\*\*/\n).*!\1!ms\" < \"$FILE\"`"
 elif [ "$EXT" == "css" -o "$EXT" == "js" ]; then #CSS/JS file
  LICENSE="`perl -0777 -pane \"s!\n*(\s*/\*\*\*.+?\*\*\*/\n).*!\1!ms\" < \"$FILE\"`"
 elif [ "$EXT" == "sh" ]; then #bash file
  LICENSE="`perl -0777 -pane \"s!#\![a-z/]+\n*(\s*###+\n.+?###\n).*!\1!ms\" < \"$FILE\"`"
 else
  echo "License file: $FILE isn't a supported file"
  return 1
 fi

 if [ "$LICENSE" == "`cat $LICENSEFILE`" ]; then #no license found
  echo "License file: $FILE doesn't contain a license"
  return 1
 fi

 echo $LICENSE

 ifs #restore IFS
 return 0
}


function setLicense()
{
 local FILE=$1
 local LICENSE=$2

 if [ ! -f "$FILE" ]; then
  echo "License file: $FILE not found"
  return 1
 fi

 EXT=`echo "$FILE" | perl -pane "s/.*\.([a-z]+)/\1/"` #get file extension

 #convert license
 if [ "$EXT" == "sh" ]; then #bash file
  LICENSE=`echo "$LICENSE" | perl -pane "s!(/\*|\*/)!##!g"` #replace /* */ comment markers with #
  LICENSE=`echo "$LICENSE" | perl -pane "s!(\*)!#!g"` #replace * comment markers with #
 fi

 #check if license has changed
 LICENSECURRENT=`getLicense "$FILE"` #get current license from file
 if [ $? == 0 ]; then #license found
  if [ "$LICENSECURRENT" == "$LICENSE" ]; then #file up-to-date
   return 0 #done
  fi
 fi

 #update file
 if [ "$EXT" == "php" ]; then #PHP file
 {
  if [ "`head -n 1 "$FILE" | grep \"<\(\?=\|[^\?]\)\"`" != "" ]; then #<?=$something?>
   perl -0777 -pane "s!!<?\n\n$LICENSE\n\n\n?>\n!ms" -i "$FILE"
  else
   perl -0777 -pane "s!(\<\?)\n*(\s*/\*\*\*.+?\*\*\*/\n+)?!\1\n\n$LICENSE\n\n\n!ms" -i "$FILE"
  fi
 }
 elif [ "$EXT" == "css" -o "$EXT" == "js" ]; then #CSS/JS file
  perl -0777 -pane "s!\n*(\s*/\*\*\*.+?\*\*\*/\n+)?!\n\n$LICENSE\n\n\n!ms" -i "$FILE"
 elif [ "$EXT" == "sh" ]; then #bash file
  perl -0777 -pane "s!(#\![a-z/]+)\n*(\s*###+\n.+?###\n+)?!\1\n\n$LICENSE\n\n\n!ms" -i "$FILE"
 else
  echo "License file: $FILE isn't a supported file"
  return 1
 fi

 return 0
}


function progress()
{
 local CURRENT=$1
 local TOTAL=$2

 #remove previous percentage
 if [ "$_PERCENTAGE" != "" -a "$_PERCENTAGE" != "100" ]; then
  for ((X=0;X<=${#_PERCENTAGE};X++)); do
   echo -en "\b"
  done
 fi

 #calculate and print new percentate
 _PERCENTAGE=`echo "($CURRENT/$TOTAL)*100" | bc -l | perl -pane "s/(.*)\..*/\1/"`
 if [ "$_PERCENTAGE" == "" ]; then _PERCENTAGE=0; fi

 echo -en "$_PERCENTAGE%"
}
